/*
 * ioGame
 * Copyright (C) 2021 - 2023  渔民小镇 （262610965@qq.com、luoyizhu@gmail.com） . All Rights Reserved.
 * # iohao.com . 渔民小镇
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package com.iohao.mmo.equip.service;

import com.iohao.mmo.common.config.GameCode;
import com.iohao.mmo.equip.entity.BasicEquipProperty;
import com.iohao.mmo.equip.entity.Equip;
import com.iohao.mmo.equip.entity.EquipTemplate;
import com.iohao.mmo.equip.entity.Goods;
import com.iohao.mmo.equip.utils.RandomUtils;
import jakarta.annotation.Resource;
import lombok.AllArgsConstructor;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.stereotype.Service;

/**
 * @author 唐斌
 * @date 2023-07-30
 * @description: 装备属性实现类
 */
@Service
@AllArgsConstructor
public class EquipTemplateService {

    @Resource
    private GoodsService goodsService;

    final MongoTemplate mongoTemplate;

    public EquipTemplate findById(String id) {

        return  mongoTemplate.findById(id, EquipTemplate.class);
    }

    public void saveEquipTemplate(EquipTemplate equipTemplate) {
        mongoTemplate.save(equipTemplate);
    }


    public void initEquipTemplate() {
        // 初始化一些装备配置，后期将移到 excel 中做。
        EquipTemplate equipTemplate = new EquipTemplate();
        equipTemplate.setPosition(1);
        equipTemplate.setLevel(10);
        equipTemplate.setTotalAttrMin(10);
        equipTemplate.setTotalAttrMax(40);

        Goods goods = new Goods();
        goods.setName("布甲");
        goods.setDesc("一件普通的布甲");
        goods.setIcon("https://www.baidu.com/img/PCtm_d9c8750bed0b3c7d089fa7d55720d6cf.png");
        goods.setImage("https://www.baidu.com/img/PCtm_d9c8750bed0b3c7d089fa7d55720d6cf.png");
        goods.setBindPrice(120);
        goods.setPrice(100);
        goods.setDealFlag(true);
        goods.setType(3);
        goodsService.save(goods);
        equipTemplate.setGoods(goods);

        mongoTemplate.insert(equipTemplate);

        // 初始化一些装备配置，后期将移到 excel 中做。
        EquipTemplate equipTemplate2 = new EquipTemplate();
        equipTemplate2.setPosition(2);
        equipTemplate2.setLevel(1);
        equipTemplate2.setTotalAttrMin(5);
        equipTemplate2.setTotalAttrMax(18);

        Goods goods2 = new Goods();
        goods2.setName("木剑");
        goods2.setDesc("一把木剑，看起来像玩具，打起来还有点疼");
        goods2.setIcon("https://www.baidu.com/img/PCtm_d9c8750bed0b3c7d089fa7d55720d6cf.png");
        goods2.setImage("https://www.baidu.com/img/PCtm_d9c8750bed0b3c7d089fa7d55720d6cf.png");
        goods2.setBindPrice(80);
        goods2.setPrice(60);
        goods2.setDealFlag(true);
        goods2.setType(3);
        goodsService.save(goods2);
        equipTemplate2.setGoods(goods2);

        mongoTemplate.insert(equipTemplate2);
    }

    /**
     * 根据装备模板随机一件新的装备
     * @param equipTemplateId
     * @return
     */
    public Equip randomEquip(String equipTemplateId,long userId){
        //取到装备模板
        EquipTemplate equipTemplate = mongoTemplate.findById(equipTemplateId, EquipTemplate.class);

        GameCode.objNotFound.assertTrue(equipTemplate!=null);

        Equip equip = new Equip();
        //随机属性值
        equip.setAttrTotal(RandomUtils.randomFromInt(equipTemplate.getTotalAttrMin(),equipTemplate.getTotalAttrMax()));
        equip.setName(equipTemplate.getGoods().getName());
        //新建装备对应的物品类
        Goods goods = equipTemplate.getGoods();
        goods.setId(null);
        goods.setType(0);
        goodsService.save(goods);
        //物品id
        equip.setItemId(goods.getId());
        equip.setUndistributedAttr(equip.getAttrTotal());
        equip.setEquipTemplate(equipTemplate);
        //初始化分配的具体属性点
        equip.setBasicEquipProperty(BasicEquipProperty.resetBasicEquipProperty());
        equip.setUserId(userId);
        mongoTemplate.save(equip);
        return equip;
    }
}
