/*
 * ioGame
 * Copyright (C) 2021 - 2023  渔民小镇 （262610965@qq.com、luoyizhu@gmail.com） . All Rights Reserved.
 * # iohao.com . 渔民小镇
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package com.iohao.mmo.person.service;

import com.iohao.mmo.hero.service.HeroService;
import com.iohao.mmo.person.entity.BasicProperty;
import com.iohao.mmo.person.entity.Hero;
import com.iohao.mmo.person.entity.Person;
import lombok.AllArgsConstructor;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Objects;

/**
 * @author 渔民小镇
 * @date 2023-07-24
 */
@Service
@AllArgsConstructor
public class PersonService {
    final MongoTemplate mongoTemplate;
    final HeroService heroService;

    public void initPerson(long userId) {
        Query query = new Query(Criteria.where("userId").is(userId));
        Person person = mongoTemplate.findOne(query, Person.class);

        if (Objects.nonNull(person)) {
            return;
        }

        createPerson(userId);
    }

    private void createPerson(long userId) {
        // 人物基础属性
        BasicProperty basicProperty = getInitBasicProperty();

        Hero firstHero = heroService.getFirstHero();

        BasicProperty plus = basicProperty.plus(firstHero.getBasicProperty());

        firstHero.setBasicProperty(plus);

        Person person = new Person();
        person.setId(userId);
        person.setBasicProperty(basicProperty);
        person.setHeroList(List.of(firstHero));
        person.setCurrentHero(firstHero);

        mongoTemplate.save(person);
    }

    public Person getPersonById(long userId) {
        return mongoTemplate.findById(userId, Person.class);
    }

    private BasicProperty getInitBasicProperty() {
        BasicProperty basicProperty = new BasicProperty();
        basicProperty.setHp(100);
        basicProperty.setMp(100);
        basicProperty.setPhysicsAttack(20);
        basicProperty.setPhysicsDefense(10);

        basicProperty.setMagicAttack(20);
        basicProperty.setMagicDefense(10);

        basicProperty.setTreatAttack(20);
        basicProperty.setSealAttack(20);
        basicProperty.setSealDefense(10);
        basicProperty.setSpeed(50);
        basicProperty.setAnger(0);
        return basicProperty;
    }
}
