/*
 * ioGame
 * Copyright (C) 2021 - 2023  渔民小镇 （262610965@qq.com、luoyizhu@gmail.com） . All Rights Reserved.
 * # iohao.com . 渔民小镇
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package com.iohao.mmo.bag.client;

import com.iohao.game.action.skeleton.protocol.wrapper.BoolValue;
import com.iohao.game.external.client.AbstractInputCommandRegion;
import com.iohao.game.external.client.command.InputRequestData;
import com.iohao.game.external.client.kit.ScannerKit;
import com.iohao.game.external.client.kit.SplitParam;
import com.iohao.mmo.bag.ItemIdConst;
import com.iohao.mmo.bag.cmd.BagCmd;
import com.iohao.mmo.bag.proto.BagItemMessage;
import com.iohao.mmo.bag.proto.BagMessage;
import com.iohao.mmo.bag.proto.UseMessage;
import com.iohao.mmo.common.provide.kit.JsonKit;
import com.iohao.mmo.common.snow.SnowKit;
import lombok.extern.slf4j.Slf4j;

import java.util.List;

/**
 * @author 渔民小镇
 * @date 2023-08-04
 */
@Slf4j
public class BagInputCommandRegion extends AbstractInputCommandRegion {
    @Override
    public void initInputCommand() {
        this.inputCommandCreate.cmd = BagCmd.cmd;
        request();
        useRequest();
    }

    @Override
    public void loginSuccessCallback() {
        // 添加一些经验值道具
        var bagItemMessage = BagInternalHelper.ofBagItemMessage(ItemIdConst.expId);
        bagItemMessage.quantity = 10;
        log.info("添加 {} 个经验值道具 {}", bagItemMessage.quantity, bagItemMessage);
        ofRequestCommand(BagCmd.incrementItem).request(bagItemMessage);
    }

    private void request() {
        ofCommand(BagCmd.bag).callback(BagMessage.class, result -> {
            BagMessage value = result.getValue();
            log.info("{}", JsonKit.toJsonString(value));
        }).setDescription("查询玩家背包");

        InputRequestData inputRequestData = () -> {
            ScannerKit.log(() -> log.info("输入【1】表示添加一个可叠加的物品。"));
            String inputType = ScannerKit.nextLine("1");

            String id = "1";
            String itemId = "1";

            if (!"1".equals(inputType)) {
                id = SnowKit.nextToString();
            }

            BagItemMessage bagItemMessage = new BagItemMessage();
            bagItemMessage.id = id;
            bagItemMessage.itemId = itemId;
            bagItemMessage.quantity = 1;

            ScannerKit.log(() -> log.info("{}", bagItemMessage));

            return bagItemMessage;
        };

        ofCommand(BagCmd.incrementItem).callback(BagItemMessage.class, result -> {
            var value = result.getValue();
            log.info("value : {}", value);
            // 重新查询一次背包
            ofRequestCommand(BagCmd.bag).request();
        }).setDescription("往背包添加（增加）物品").setInputRequestData(inputRequestData);

        inputRequestData = () -> {
            ScannerKit.log(() -> log.info("输入需要减少的物品信息，格式 [背包物品id-数量]"));
            String inputType = ScannerKit.nextLine("1-1");

            SplitParam param = new SplitParam(inputType);
            String id = param.getString(0);
            int quantity = param.getInt(1, 1);

            BagItemMessage bagItemMessage = new BagItemMessage();
            bagItemMessage.id = id;
            bagItemMessage.quantity = quantity;

            ScannerKit.log(() -> log.info("{}", bagItemMessage));

            return bagItemMessage;
        };

        ofCommand(BagCmd.decrementItem).callback(BagItemMessage.class, result -> {
            var value = result.getValue();
            log.info("value : {}", value);

            // 重新查询一次背包
            ofRequestCommand(BagCmd.bag).request();
        }).setDescription("从背包减少物品").setInputRequestData(inputRequestData);


    }

    private void useRequest() {
        InputRequestData inputRequestData = () -> {
            ScannerKit.log(() -> log.info("输入需要使用的背包物品，格式 [背包物品id-数量]"));
            String inputType = ScannerKit.nextLine("1-1");

            SplitParam param = new SplitParam(inputType);
            String id = param.getString(0);
            int quantity = param.getInt(1, 1);

            var useItemMessage = BagInternalHelper.ofUseItemMessage(id);
            useItemMessage.quantity = quantity;

            UseMessage useMessage = new UseMessage();
            useMessage.useItems = List.of(useItemMessage);
            return useMessage;
        };

        ofCommand(BagCmd.use).callback(BoolValue.class, result -> {
            var value = result.getValue();
            log.info("value : {}", value);
        }).setDescription("使用背包物品").setInputRequestData(inputRequestData);
    }
}