/*
 * ioGame
 * Copyright (C) 2021 - 2023  渔民小镇 （262610965@qq.com、luoyizhu@gmail.com） . All Rights Reserved.
 * # iohao.com . 渔民小镇
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package com.iohao.mmo.equip.client;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONWriter;
import com.iohao.game.action.skeleton.protocol.wrapper.ByteValueList;
import com.iohao.game.action.skeleton.protocol.wrapper.StringValue;
import com.iohao.game.external.client.AbstractInputCommandRegion;
import com.iohao.game.external.client.command.InputRequestData;
import com.iohao.game.external.client.kit.ScannerKit;
import com.iohao.mmo.common.provide.kit.JsonKit;
import com.iohao.mmo.equip.cmd.EquipCmd;
import com.iohao.mmo.equip.proto.*;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.List;

/**
 * @author 渔民小镇
 * @date 2023-07-30
 */
@Slf4j
public class EquipInputCommandRegion extends AbstractInputCommandRegion {
    @Override
    public void initInputCommand() {
        this.inputCommandCreate.cmd = EquipCmd.cmd;

        request();

        listen();
    }

    private void listen() {

    }

    private void request() {
        InputRequestData inputRequestData;

        // 8-1
        ofCommand(EquipCmd.getEquipList).callback(ByteValueList.class, result -> {
            List<EquipMessage> value = result.toList(EquipMessage.class);
            log.info("装备列表信息 : {}", JsonKit.toJsonString(value));
        }).setDescription("获取装备列表信息");

        // 动态请求参数（参数值）
        inputRequestData = () -> {
            ScannerKit.log(() -> log.info("请输入要查询的装备id"));
            String id = ScannerKit.nextLine();

            // 请求参数
            StringValue stringValue = StringValue.of(id);

            return stringValue;
        };

        // 8-2
        ofCommand(EquipCmd.getEquip).callback(EquipMessage.class, result -> {
            EquipMessage value = result.getValue();
            log.info("装备信息 : {}", value);
        }).setDescription("查询装备信息").setInputRequestData(inputRequestData);

        // 动态请求参数（参数值）
        inputRequestData = () -> {
            ScannerKit.log(() -> log.info("请输入装备id"));
            String id = ScannerKit.nextLine();
            ScannerKit.log(() -> log.info("请输入体质"));
            int constitution = ScannerKit.nextInt(0);
            ScannerKit.log(() -> log.info("请输入魔力"));
            int magicPower = ScannerKit.nextInt(0);
            ScannerKit.log(() -> log.info("请输入力量"));
            int power = ScannerKit.nextInt(0);
            ScannerKit.log(() -> log.info("请输入耐力"));
            int endurance = ScannerKit.nextInt(0);
            ScannerKit.log(() -> log.info("请输入敏捷"));
            int agile = ScannerKit.nextInt(0);

            BasicEquipPropertyMessage basicEquipPropertyMessage = new BasicEquipPropertyMessage(constitution,magicPower,power,endurance,agile);
            EquipMessage equipMessage = new EquipMessage();
            equipMessage.basicEquipProperty = basicEquipPropertyMessage;
            equipMessage.id = id;
            return equipMessage;
        };

        // 8-3
        ofCommand(EquipCmd.allotEquip).callback(EquipMessage.class, result -> {
            EquipMessage value = result.getValue();
            log.info("装备属性信息 : {}", value);
        }).setDescription("分配装备属性信息").setInputRequestData(inputRequestData);


        // 动态请求参数（参数值）
        inputRequestData = () -> {
            ScannerKit.log(() -> log.info("请输入装备id"));
            String equipId = ScannerKit.nextLine();
            ScannerKit.log(() -> log.info("请输入道具id"));
            String propId = ScannerKit.nextLine();

            EquipResetMessage equipResetMessage = new EquipResetMessage();
            equipResetMessage.equipId = equipId;
            equipResetMessage.propId = propId;
            return equipResetMessage;
        };

        // 8-4
        ofCommand(EquipCmd.resetEquip).callback(EquipMessage.class, result -> {
            EquipMessage value = result.getValue();
            log.info("装备属性信息 : {}", value);
        }).setDescription("重新随机总属性点").setInputRequestData(inputRequestData);

        // 8-10
        ofCommand(EquipCmd.initEquipTemplate).callback(EquipTemplateMessage.class, result -> {
            EquipTemplateMessage value = result.getValue();
            log.info("装备模板信息 : {}", value);
        }).setDescription("初始化装备模板");

        // 动态请求参数（参数值）
        inputRequestData = () -> {
            String ids = "64d0838a6f0c894087cab868,64d0838a6f0c894087cab86a";

            // 请求参数
            StringValue stringValue = StringValue.of(ids);

            return stringValue;
        };
        // 8-13
        ofCommand(EquipCmd.randomEquip).callback(EquipMessage.class, result -> {
            EquipMessage value = result.getValue();
            log.info("装备信息 : {}", value);
        }).setDescription("根据装备模板列表各随机出一件新的装备").setInputRequestData(inputRequestData);
    }
}
